/***************************************************************************************
*     Programa exemplo para o uso do Leitor RFID RDM630                                *
*     Compre o leitor em nosso site www.msseletronica.com - MSS Eletrnica             *
*     Desenvolvido por Misael - 26/08/2012                                             *
*                                                                                      *
*     Introduo:                                                                      *
*                                                                                      *
*     Como  um programa exemplo, no me preocupei em fazer um programa                *
*  "eficinte" e sim funcional de simples entendimento                                 *
*     Neste programa tentei usar o mnimo de informaes/funes possivel              *
*                                                                                      *
*     Objetivo do programa:                                                            *
*                                                                                      *
*     - Possibilitar memorizar o ID de um carto RFID atravs do acionamento de um     *
*  boto:                                                                              *
*        Pressionando o boto, um LED_OK acende esperando o carto, assim que ler      *
*       o carto, pisca este mesmo LED_OK informando que terminou de memorizar         *
*     - Um LED_OK dever acender sempre que o carto de ID memorizado for lido         *
*     - Um LED_ERR dever acender sempre que outro carto for lido                     *
*     - Os dados do carto so recebidos na interrupo da porta serial                *
****************************************************************************************/

/***************************************************************************************
*     PIC usado: PIC16F628A                                                            *
*                                                                                      *
*     GRAVA_ID -> Pino RB5                                                             *
*     LED_OK   -> Pino RB4                                                             *
*     LED_ERR  -> Pino RB3                                                             *
*     TX       -> Pino RB2                                                             *
*     RX       -> Pino RB1                                                             *
*                                                                                      *
*     Oscilador -> 4Mhz (XT)                                                           *
*     Master Clear -> ON                                                               *
****************************************************************************************/
#include "H:\MISAEL\Anuncios ML\RFID 125khz - 2 UART\Exemplos\pic16f628a_rfid.h"
  #ZERO_RAM

int buffer[12];   //declarando um array que receber o buffer enviado pelo leitor RFID
int id_cartao[12];//declarando um array para receber o id do carto memorizado, para um programa melhor, estes dados deveriam ser memorizados na EEPROM

int dado = 0;  //Recebe temporariamente um byte recebido pela porta serial (Enviado pelo Leitor RFID)
int cont = 0;  //variavel que conta o nmero de bytes recebidos
int aux = 0;   //variavel de uso geral
short buffer_iniciado = 0; //indica que um novo buffer foi iniciado (Comeou a receber um novo pacote de dados do RFID)
short buffer_finalizado = 0; //Indica que tem um novo buffer cheio e que j pode ser tratado
short aguardando_id = 0;   //Indica que o boto foi pressionado e que esta aguardando o ID a ser memorizado
short id_gravado = 0;  //Informa quando o ID do carto j foi gravado
short id_errado = 0; //Informa que o ultimo ID lido no  o do cartao memorizado

#byte PORTB = 0x06
#byte PORTA = 0x05
#byte CMCON = 0x1F   //Byte de configurao dos pinos Analogicos/digitais do PORTA

#use delay(clock=4000000)  //Definindo a frequencia de clock para o Delay
#use fast_io(a)   //Controle de direo do PORTA pelo programa
#use fast_io(b) //Controle de direo do PORTB pelo programa

#bit LED_OK = PORTB.4   //LED que mostra que o ID esta correto
#bit LED_ERR = PORTB.3 //LED que indica que o ID esta incorreto
#bit GRAVA_ID = PORTB.5 //Pino onde dever ser ligado o boto para o PIC gravar o ID do carto (Este pino ser lido em nivel alto)

#int_RDA //Inicio da interrupo da recepo Serial
void  RDA_isr(void) 
{

dado = getch();//Le do dado recebido pela porta serial (Vindo do RFID)

if (aguardando_id)//Testa se esta aguardando o ID a ser memorizado
   {
   if (dado == 0x02)//teste se  um byte de inicio do buffer (O RFID envia o valor 0x02 como inicio e 0x03 como final)
      {
      buffer_iniciado = 1;//Indica que o buffer foi iniciado
      cont = 0;//Reinicia o contador de bytes recebidos
      }
      else if (cont >= 11)//Testa se j recebeu os 12 bytes de identificao do carto (10 bytes de ID, 1 de CheckSun e o outro no identifiquei)
               {
               id_gravado = 1;//Informa que j recebeu todo o pacote como o ID do carto a ser memorizado
               LED_OK = 0;
               aguardando_id = 0;//Limpa o flag que indica que esta esperando o ID para ser memorizado
               }
               else  //se ainda no recebeu todos os dados, grava o dado no buffer do ID do carto a ser memorizado
                  {
                  id_cartao[cont] = dado;//grava o dado do buffer do ID do carto a ser memorizado
                  cont = cont + 1;//incrementa a contagem
                  }

     
   }
   else //se no esta aguardando o ID a ser memorizado, grava o dado no buffer para ser testado e comparado com o ID memorizado
      {
      if (dado == 0x02)//teste se  um byte de inicio do buffer (O RFID envia o valor 0x02 como inicio e 0x03 como final)
         {
         buffer_iniciado = 1;//Indica que o buffer foi iniciado
         cont = 0;//Reinicia o contador de bytes recebidos
         }
         else if (cont >= 11)//Testa se j recebeu os 12 bytes de identificao do carto (10 bytes de ID, 1 de CheckSun e o outro no identifiquei)
                  {
                  buffer_finalizado = 1;//Indica que tem um novo buffer para ser tratado (Comparar os IDs)
                  }
                  else//Se ainda no recebeu todos os dados, grava o dado no buffer para comparao futura do ID
                     {
                     buffer[cont] = dado;//grava o dado no buffer
                     cont = cont + 1;//incrementa a contagem
                     }
     
      }

}
//Fim da interrupo da porta serial


void main()
{

   port_b_pullups(TRUE);
   setup_timer_0(RTCC_INTERNAL|RTCC_DIV_1);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   enable_interrupts(INT_RDA);
   enable_interrupts(GLOBAL);
   //Setup_Oscillator parameter not selected from Intr Oscillator Config tab

   // TODO: USER CODE!!
   
   //configurando os pinos
   CMCON = 0b00000111;//configurando todos os pinos do PORTA como pinos digitais
   set_tris_b(0b00100010);//PINO B5 como entrada (chave para memorizar o ID) o restante saida, menos o pino RX (B1)
   PORTB = 0;//Inicia com todos os lEDs desligados
   
   while(1)
     {
     if(id_gravado)//Testa se j recebeu todo o ID do carto a ser memorizado, se sim indica piscando o LED_OK
         {
         LED_OK = 0;
         delay_ms(1000);
         LED_OK = 1;
         delay_ms(1000);
         LED_OK = 0;
         delay_ms(1000);
         id_gravado = 0;
         }
         
     if((buffer_finalizado)&&(!aguardando_id))//Testa se tem um buffer finalizado e que no esta aguardando o ID a ser memorizado
         {
         aux = 0;
         id_errado = 0;
         while(aux < cont)
               {
               if(buffer[aux] != id_cartao[aux])//Se o buffer esta finalizado, compara o ID lido com o ID memorizado
                  {
                  id_errado = 1;//Se o ID  diferente, seta o Flag
                  }
               aux = aux + 1;
               }
                 
         if(id_errado)//Se o ID esta errado, pisca o LED_ERR
            {
            LED_ERR = 1;
            delay_ms(1000);
            LED_ERR = 0;
            delay_ms(1000);
            }
            else //Se o ID esta correto pisca o LED_OK
               {
               LED_OK = 1;
               delay_ms(1000);
               LED_OK = 0;
               delay_ms(1000);
               }
         buffer_iniciado = 0;//Inicializa os flags
         buffer_finalizado = 0;//Inicializa os flags
         }
         
      if(GRAVA_ID)//Testa se o boto foi pressionado
         {
         delay_ms(100);//Espera um tempo e testa o boto novamente (Isto  um debouse mau feito)
         if(GRAVA_ID)//Se aps os 100ms o boto continuar pressionado, inicia o processo para aguarda o ID a ser memorizado
            {
            LED_OK = 1;//Informa que pode passar o carto
            aguardando_id = 1;//Informa que esta aguardando um novo ID (Ser lido na interrupo da porta serial)
            while(GRAVA_ID){}//Aguarda o boto ser solto
            }
    
         }
      }


}
